

#ifndef APP_H_
#define APP_H_

/* bit operation */
#define SETBIT(x,y) (x |= _BV(y))
#define CLRBIT(x,y) (x &=~ _BV(y))
#define CHKBIT(x,y) (x & _BV(y))
#define INVBIT(x,y) (x ^= _BV(y))

/* 基本定时时间(毫秒) */
#define TIME_MS  100
#define OCR_VALUE 	(TIME_MS * (F_CPU / 1000000UL) / (1024 / 1000.0f))

/* 连接等待 */
//#define WAIT_C_CNT 	(TIME_WAIT_C * 1000 / TIME_MS)
#define WAIT_C_CNT 	(TIME_WAIT_C * 10)

/* 数据等待 */
//#define WAIT_D_CNT 	(TIME_WAIT_D * 1000 / TIME_MS)
#define WAIT_D_CNT 	(TIME_WAIT_D * 10)

/* 是否倍速 1: BUADRATE*2  0:BUADRATE*1 */
#define U2X_ 		0


#define GLUE_2(a,b)  a##b
#define PORT(x) GLUE_2(PORT,x)
#define DDR(x)  GLUE_2(DDR,x)
#define PIN(x)  GLUE_2(PIN,x)

#define SET_LED_PORT_OUT SETBIT(DDR(LED_PORT), LED_PIN)
#define LED_ACT		INVBIT(PORT(LED_PORT), LED_PIN)

#define SET_TRI_PORT_IN CLRBIT(DDR(TRI_PORT), TRI_PIN);SETBIT(PORT(TRI_PORT), TRI_PIN)
#define CHECK_TRI_PIN 	CHKBIT(PIN(TRI_PORT), TRI_PIN)	

#define GLUE_3(a,b,c) a##b##c
#define UBRRH_R(x) GLUE_3(UBRR, x, H)
#define UBRRL_R(x) GLUE_3(UBRR, x, L)
#define UCSRA_R(x) GLUE_3(UCSR, x, A)
#define UCSRB_R(x) GLUE_3(UCSR, x, B)
#define UCSRC_R(x) GLUE_3(UCSR, x, C)
#define UDR_R(x)   GLUE_2(UDR, x)

#ifdef URSEL /* 如M16用URSEL位来决定读写UCSRC或UBRRH */
#define SET_URSEL _BV(URSEL)
#else
#define SET_URSEL 0
#endif

#define UBRRH_REG UBRRH_R(UART_NO)
#define UBRRL_REG UBRRL_R(UART_NO)
#define UCSRA_REG UCSRA_R(UART_NO)
#define UCSRB_REG UCSRB_R(UART_NO)
#define UCSRC_REG UCSRC_R(UART_NO)
#define UDR_REG   UDR_R(UART_NO)

#define CHECK_RXC (UCSRA_REG & _BV(RXC))

#if U2X_
#define	UBRR_VALUE (uint16_t)(F_CPU / (8UL * BUADRATE) - 1)
#else
#define UBRR_VALUE (uint16_t)(F_CPU / (16UL * BUADRATE) - 1)
#endif

#define UBRRH_VALUE (uint8_t)(UBRR_VALUE / 256)
#define UBRRL_VALUE (uint8_t)(UBRR_VALUE % 256)


/* XMODEM控制字 */
#define XMODEM_NUL 0x00
#define XMODEM_SOH 0x01
#define XMODEM_STX 0x02
#define XMODEM_EOT 0x04
#define XMODEM_ACK 0x06
#define XMODEM_NAK 0x15
#define XMODEM_CAN 0x18
#define XMODEM_EOF 0x1a
#define XMODEM_RWC 'C'


extern void device_init(void);
extern void timer_init(void);
extern void usart_init(void);
extern void write_udr(uint8_t c);
extern uint8_t read_udr(void);
extern void put_string(char *str);
extern void quit(void);
extern void boot_program_page(uint32_t page, uint8_t *buf);

#endif
