
//lcd.c mp3 bootloader

#include <avr/io.h>
#include <avr/pgmspace.h>

#include "lcd.h"

prog_uchar font_6_8[][6] PROGMEM = {
    { 0x00, 0x7F, 0x49, 0x49, 0x49, 0x36 },   // B
    { 0x00, 0x38, 0x44, 0x44, 0x44, 0x38 },   // o
    { 0x00, 0x38, 0x44, 0x44, 0x44, 0x38 },   // o
    { 0x00, 0x04, 0x3F, 0x44, 0x40, 0x20 },   // t
    { 0x00, 0x00, 0x41, 0x7F, 0x40, 0x00 },   // l
    { 0x00, 0x38, 0x44, 0x44, 0x44, 0x38 },   // o
    { 0x00, 0x20, 0x54, 0x54, 0x54, 0x78 },   // a
    { 0x00, 0x38, 0x44, 0x44, 0x48, 0x7F },   // d
    { 0x00, 0x38, 0x54, 0x54, 0x54, 0x18 },   // e
    { 0x00, 0x7C, 0x08, 0x04, 0x04, 0x08 }    // r
};

/* bit operation */
#define SETBIT(x,y) (x |= _BV(y))
#define CLRBIT(x,y) (x &=~ _BV(y))
#define CHKBIT(x,y) (x & _BV(y))
#define INVBIT(x,y) (x ^= _BV(y))

void spi_init(void)
{
    //MISO 设为输入
    //MOSI，SCK 设为输出，SS 也设为输出
    //SS（PB0）脚为SD 卡CS 脚
    //SPI_PORT |= (_BV(SPI_MOSI)|_BV(SPI_SCK)|_BV(SPI_SS)|_BV(SPI_MISO));
    SPI_DDR |= (_BV(SPI_MOSI)|_BV(SPI_SCK)|_BV(SPI_SS));
    //SPI_DDR &=~ _BV(SPI_MISO);

    //使能 SPI主机模式,设置时钟速率为 fck/128
    SPCR = _BV(SPE)|_BV(MSTR)|_BV(SPR1)|_BV(SPR0);
    SPSR &=~ _BV(SPI2X) ;
}

unsigned char spi_access_byte(unsigned char d)
{
    SPDR = d;
    do ;
    while(!CHKBIT(SPSR, SPIF)); 
    return SPDR;
}

void lcd_power_on_reset(void)
{
    //LCD_CTRL_PORT |= (_BV(LCD_CE)|_BV(LCD_DC)|_BV(LCD_RST));
    LCD_CTRL_DDR |= (_BV(LCD_CE)|_BV(LCD_DC)|_BV(LCD_RST));

    CLRBIT(LCD_CTRL_PORT, LCD_CE);
    CLRBIT(LCD_CTRL_PORT, LCD_RST);
    asm volatile(
	    "nop" "\n\t"
	    "nop" "\n\t"
	    : : );
    SETBIT(LCD_CTRL_PORT, LCD_RST);
    SETBIT(LCD_CTRL_PORT, LCD_CE);
}

void lcd_write_byte(unsigned char d, unsigned char rs)
{
    CLRBIT(LCD_CTRL_PORT, LCD_CE);

    if(rs ==  CMD) {
	CLRBIT(LCD_CTRL_PORT, LCD_DC);
    } else {
	SETBIT(LCD_CTRL_PORT,LCD_DC);//DC=1select data register
    }
    spi_access_byte(d);

    SETBIT(LCD_CTRL_PORT, LCD_CE);
}

void lcd_init(void)
{
    spi_init();
    lcd_power_on_reset();
    lcd_write_byte(0x21,CMD);
    //lcd_write_byte(0x80|30,CMD);//?
    lcd_write_byte(0x06,CMD);
    lcd_write_byte(0x13,CMD);
    lcd_write_byte(0x20,CMD);
    lcd_write_byte(0x0c,CMD);
}

void disp(void)
{
    unsigned char col;
    unsigned char c;

    for(col = 0; col < 6; col++){ //共6行

	lcd_write_byte(0x80|0, CMD);//x
	lcd_write_byte(0x40|col, CMD);//y

	for(c = 0; c < 84; c ++)
	    lcd_write_byte(0, DATA);//每行84列
    }
    lcd_write_byte(0x80|0, CMD);//x
    lcd_write_byte(0x40|0, CMD);//y

    for(c = 0; c < 10; c++)
	for(col = 0; col < 6; col++)
	    lcd_write_byte(pgm_read_byte(&font_6_8[c][col]), DATA);
}

/* end of file */

